<?php

require '../../vendor/autoload.php';

require 'log.php';

include 'config.php';



use Lcobucci\JWT\Builder;

use Lcobucci\JWT\Signer\Hmac\Sha256;

use Lcobucci\JWT\ValidationData;

use Lcobucci\JWT\Parser;

use PHPMailer\PHPMailer\PHPMailer;

use PHPMailer\PHPMailer\Exception;



$app = new Slim\App(["settings" => $config]);



$lrPluginUrl = 'https://leonardofifthavenue.com/wp-content/plugins/LeonardosRental/';

$lcmPluginUrl = 'https://leonardofifthavenue.com/wp-content/plugins/LeonardosCustomMade/';

$wpUploadUrl = 'https://leonardofifthavenue.com/wp-content/uploads/';

$wpUploadUrl_dev = 'http://ecomproject.leonardos.com/wp-content/uploads/';



//Handle Dependencies

$container = $app->getContainer();


$container['apidb'] = function ($c) {

   

   try {

       $db = $c['settings']['apidb'];

       $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );

       $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);

       return $pdo;

   }

   catch(\Exception $ex) {

       return $ex->getMessage();

   }

};



$container['webstoredb_dev'] = function ($c) {

   

    try{

        $db = $c['settings']['webstoredb_dev'];

        $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );

        $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);

        return $pdo;

    }

    catch(\Exception $ex) {

        return $ex->getMessage();

    }

 };



 $container['webstoredb'] = function ($c) {

   

    try{

        $db = $c['settings']['webstoredb'];

        $options  = array(PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION, PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC, );

        $pdo = new PDO("mysql:host=" . $db['servername'] . ";dbname=" . $db['dbname'], $db['username'], $db['password'],$options);

        return $pdo;

    }

    catch(\Exception $ex) {

        return $ex->getMessage();

    }

 };



 $mw_authorize = function( $request, $response, $next ) {

    $auth_header = $request->getHeaderLine('authorization');



    $auths = explode(',', $auth_header );

    $auth_token;



    if(count($auths) < 2)

        return $response->withJson( array( 'status' => false, 'message' => 'Access Denied.', 'signout' => true ), 401 );



    $auth_token = str_replace('Bearer ', '', $auths[1]);

    $token = '';



    // Validate the user token

    if ( isset( $auth_token ) ) {

        try {

            $token = (new Parser())->parse((string) $auth_token); // Parses from a string

            $token->getHeaders(); // Retrieves the token header

            $token->getClaims(); // Retrieves the token claims

            

            $usr = $token->getClaim('usr');

            $prv = $token->getClaim('prv');



            $data = new ValidationData(); // It will use the current time to validate (iat, nbf and exp)

            $data->setIssuer('https://m2mkiosk.leonardofifthavenue.com');

            $data->setAudience('https://m2mkiosk.leonardofifthavenue.com/admin');

            $data->setId('4f1g23a12aa');

            

            if ( !$token->validate($data) )

                return $response->withJson( array( 'status' => false, 'message' => 'Access Denied.', 'signout' => true ), 401 );



            // generate new token with new expiration date

            $signer = new Sha256();

            $token = (new Builder())->setIssuer('https://m2mkiosk.leonardofifthavenue.com') // Configures the issuer (iss claim)

                ->setAudience('https://m2mkiosk.leonardofifthavenue.com/admin') // Configures the audience (aud claim)

                ->setId('4f1g23a12aa', true) // Configures the id (jti claim), replicating as a header item

                ->setIssuedAt(time()) // Configures the time that the token was issue (iat claim)

                ->setExpiration(time() + 900 ) // Configures the expiration time of the token (exp claim)

                ->set('usr', $usr) // Configures a new claim, called "username"

                ->set('prv', $prv) // Configures a new claim, called "privileges"

                ->sign($signer, 'Lf@-m2M-K10Zk') // creates a signature using "testing" as key

                ->getToken(); // Retrieves the generated token

        }

        catch(\Exception $ex) {

            return $response->withJson( array( 'error' => true, 'message' => 'Could not authenticate.' ), 400 );

        }

    }



    $request = $request->withAttribute('token', $token->__toString());

    $response = $next($request, $response);

    return $response;

 };



// Add the authentication function to the API execution stack 

$app->add( function ( $request, $response, $next ) {

    $auth_header = $request->getHeaderLine('authorization');


    $auths = explode(',', $auth_header );

    $auth_key;



    if(count($auths) < 2)

        $auth_key = $auths[0];

    else {

        $auth_key = $auths[0];

    }



    // Validate the Authorization Key

    if ( isset( $auth_key ) ) {

		

        try{

            $con = $this->apidb;

            $sql = "SELECT id FROM tblStore where auth_key = :auth_key";

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array( ':auth_key' => $auth_key );

            $pre->execute($values);

            $result = $pre->fetch();

        }

        catch(\Exception $ex) {

            return $response->withJson( array( 'error' => true, 'message' => 'Could not authenticate.' ), 400 );

        }

		

        if ( !$result ) {

            return $response->withJson( array( 'error' => true, 'message' => 'Invalid Api key.' ), 401 );

            // $app->stop();

        } 



        $response = $next($request, $response);

        return $response;

		

    } else {

        return $response->withJson( array( 'error' => true, 'message' => 'Api key is misssing.' ), 400 );

        // $app->stop();

    }

});



// Middleware to get clients IP

$checkProxyHeaders = false; 

$trustedProxies = [];

$app->add(new RKA\Middleware\IpAddress($checkProxyHeaders, $trustedProxies));



// Let the world know that we are live!

$app->get('/', function() {

    return $response->withJson(array('status' => 'active','message'=> 'Welcome to Leonardo Fifth Avenue API.'),200);

});



// Get all the product types available for the Made to Measure Kiosk

$app->get('/prodtype', function ($request, $response) use ($lrPluginUrl) {

    try {

        

        $con = $this->webstoredb;

        $sql = "SELECT type_id, name, CONCAT('" . $lrPluginUrl . "assets/images/checkout-', LCASE(name), '.svg') img, FALSE active FROM wp_xwyq_cm_product_type order by type_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute($values);

        $result = $pre->fetchall();



        if( $result ) {

            return $response->withJson(array('status' => 'true','data'=> $result),200);

        }else {

            return $response->withJson(array('status' => 'Design To Measure module not configured.'),422);

        }

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

    

 });



 // Get all the customizable categories for the selected product type with the SVG image path

 $app->get('/modcat/{typeId}', function ($request, $response) use ($lcmPluginUrl) {

    try{

        

        $con = $this->webstoredb;

        $sql = "SELECT mod_id, type_id, name, CONCAT('" . $lcmPluginUrl . "', image_url) image_url, mod_order, active, FALSE selected, '' description FROM wp_xwyq_cm_modification WHERE active = 1 and type_id = :type_id order by mod_order";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':type_id' => $request->getAttribute('typeId') );

        $pre->execute( $values );

        $result = $pre->fetchall();



        $image = '';

        switch( intval( $request->getAttribute('typeId') ) ) {

            case 1: // Coats

                $image = file_get_contents( $lcmPluginUrl . 'assets/images/suit_layers/suit.svg' );

                break;

            case 2: // Pants

                $image = file_get_contents( $lcmPluginUrl . 'assets/images/suit_layers/pant.svg' );

                break;

            case 3: // Shirts

                $image = file_get_contents( $lcmPluginUrl . 'assets/images/suit_layers/shirt.svg' );

                break;

            case 4: // Vest

                $image = file_get_contents( $lcmPluginUrl . 'assets/images/suit_layers/vest.svg' );

                break;

        }



        if( $result ) {

            return $response->withJson(array('status' => 'true','data'=> $result, 'svg' => $image ), 200);

        }else {

            return $response->withJson(array('status' => 'There are no customization categories for this product type.'), 422);

        }

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

    

 });



  // Get all options of the customizable category

  $app->get('/modopt/{modId}', function ($request, $response) use($lcmPluginUrl) {

    try{

        

        $con = $this->webstoredb;

        $sql = "select mod_option_id, name, CONCAT('" . $lcmPluginUrl . "', image_url) image_url, option_value, FALSE selected from wp_xwyq_cm_mod_option where active = 1 and mod_id = :mod_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':mod_id' => $request->getAttribute('modId') );

        $pre->execute( $values );

        $result = $pre->fetchall();



        if( $result ) {

            return $response->withJson(array('status' => 'true','data'=> $result), 200);

        }else {

            return $response->withJson(array('status' => 'There are no customization for this category.'), 422);

        }

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

    

 });



   // Get all categories where fabrics can be applied for the given product type

   $app->get('/fabcat/{typeId}', function ($request, $response) {

    try{

        

        $con = $this->webstoredb;

        $sql = "select fabric_option_id, name, FALSE selected from wp_xwyq_cm_fabric_modification_type where active = 1 and type_id = :type_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':type_id' => $request->getAttribute('typeId') );

        $pre->execute( $values );

        $result = $pre->fetchall();



        if( $result ) {

            return $response->withJson(array('status' => 'true','data'=> $result), 200);

        }else {

            return $response->withJson(array('status' => 'There are no customization categories for this product type.'), 422);

        }

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

    

 });



// Get all fabrics configured for the fabric_option_id and product_id

$app->get('/fabmod/{fabricOptId}/{productId}', function ($request, $response) use($wpUploadUrl,$wpUploadUrl_dev) {

    try{

        

        $con = $this->webstoredb;

        $sql = "select f.fabric_id, f.name, f.image_url, FALSE selected from wp_xwyq_cm_prod_mod_fabrics mf, wp_xwyq_cm_fabrics f where mf.fabric_id = f.fabric_id and f.active = 1 and mod_id = :modId and product_id = :productId";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':modId' => $request->getAttribute('fabricOptId'), ':productId' => $request->getAttribute('productId') );

        $pre->execute( $values );

        $result = $pre->fetchall();



        $i = 0;

        foreach($result as $row) {

            $con = $this->webstoredb;

            $sql = "select meta_value from wp_xwyq_postmeta where post_id = :postId and meta_key = '_wp_attached_file'";

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array( ':postId' => $row['image_url'] );

            $pre->execute( $values );

            $inResult = $pre->fetch();



            if( count( $inResult ) > 0 ) {

                $image = pathinfo( $inResult['meta_value'] );

                $result[$i]['image_url'] = $wpUploadUrl . $image['dirname'] . '/' . $image['filename'] . '-100x100.' . $image['extension'];

            }



            $i++;

        }



        if( $result ) {

            return $response->withJson(array('status' => 'true','data'=> $result), 200);

        }else {

            return $response->withJson(array('status' => 'There are no customization categories for this product type.'), 422);

        }

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

    

 });



 $app->get('/specialcharges', function ($request, $response) {

    try{

        

        $con = $this->apidb;

        $sql = "select special_charges_id, name, description, amount from tblSpecialCharges where active = 1";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        return $response->withJson(array('status' => 'true','data'=> $result), 200);       

    }

    catch(\Exception $ex) {

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

 });



$app->post('/order', function ($request, $response) {

   

   try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        // Validate request parameters

        $errors = validateOrderParameters( $params );



        $con = $this->apidb;



        // Validate the stylist code and referral code

        $sql = "select *

                from tblRepresentative

                where rep_code = :rep_code and type = 'Stylist'";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

            ':rep_code' => $params->rep

        );

        $pre->execute($values);

        $result = $pre->fetchAll();



        if( count($result) < 1 ) // If there are no stylist with the given stylist code

            array_push( $errors, 'There are no stylist with the given code.' );



        // Referrals

        if( strlen( $params->referral ) > 0 ) {

            $sql = "select *

                    from tblRepresentative

                    where rep_code = :rep_code and type = 'Referral'";

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                ':rep_code' => $params->referral

            );

            $pre->execute($values);

            $result = $pre->fetchAll();

            if( count($result) < 1 ) // If there are no referral with the given stylist code

                array_push( $errors, 'There are no referral with the given code.' );

        }



        // If there are errors return the error messages

        if( count( $errors ) > 0 ) {

            return $response->withJson(array('status' => 'error', 'errors' => $errors), 200);

        }



        $sql = "SELECT MAX(order_id) + 1 as order_id from tblOrder";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute();

        $result = $pre->fetch();



        // Process payment

        if ( $params->paymentMethod == 'Stripe' ) {

            $curl = curl_init();

            curl_setopt_array($curl, array(

                CURLOPT_URL => "https://api.stripe.com/v1/charges",

                CURLOPT_RETURNTRANSFER => true,

                CURLOPT_ENCODING => "",

                CURLOPT_MAXREDIRS => 10,

                CURLOPT_TIMEOUT => 30,

                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,

                CURLOPT_CUSTOMREQUEST => "POST",

                CURLOPT_POSTFIELDS => "amount=" . ( round ($params->paymentAmount, 2) * 100 ) . 

                                    "&currency=usd&description=" . urlencode ( 'LFA - Design To Measure Order ' . $result['order_id'] ) . 

                                    "&source=" . $params->paymentToken,

                CURLOPT_HTTPHEADER => array(

                "Authorization: Bearer sk_live_Av8QgdpJTRmxz3V5LHpJkuxt",

                "Content-Type: application/x-www-form-urlencoded",

                "cache-control: no-cache"

                ),

            ));



            $stripe_response = curl_exec($curl);

            $err = curl_error($curl);



            curl_close($curl);



            if ($err) {

                return $response->withJson(array('status' => 'error', 'payment_error' => $err), 200);

            } 

        }

        // else, if it is the payment method is ATH / Credit Card at store, continue

        

        // Create order after payment is received

        $sql = "INSERT INTO `tblOrder` (`order_date`, 

                                        `rep_code`,

                                        `customer_name`, 

                                        `customer_lastname`, 

                                        `customer_email`, 

                                        `customer_phone`,

                                        `customer_gender`, 

                                        `customer_birthdate`, 

                                        `customer_event_code`, 
					`customer_event_date`,
                                        `customer_address_line1`, 

                                        `customer_address_line2`, 

                                        `customer_city`, 

                                        `customer_state`, 

                                        `customer_zipcode`, 

                                        `partner_fullname`, 

                                        `partner_phone`, 

                                        `partner_email`, 

                                        `subtotal`,

                                        `tax`,

                                        `total`,

                                        `discount_note`,

                                        `discount_amount`,

                                        `status`,

                                        `referral_code`,

                                        `coupon_code`,

                                        `coupon_amount_used`,

                                        `store`,
					`fitting_date`) VALUES (

                                        :order_date,

                                        :rep_code,

                                        :customer_name,

                                        :customer_lastname,

                                        :customer_email,

                                        :customer_phone,

                                        :customer_gender,

                                        :customer_birthdate,

                                        :customer_event_code,
					:customer_event_date,
                                        :customer_address_line1,

                                        :customer_address_line2,

                                        :customer_city,

                                        :customer_state,

                                        :customer_zipcode,

                                        :partner_fullname,

                                        :partner_phone,

                                        :partner_email,

                                        :subtotal,

                                        :tax,

                                        :total,

                                        :discount_note,

                                        :discount_amount,

                                        :order_status,

                                        :referral_code,

                                        :coupon_code,

                                        :coupon_amount_used,

                                        :store,
					:fitting_date)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_date' => date('Y-m-d h:i:s a', time()),

                        ':rep_code' => $params->rep,

                        ':customer_name' => $params->customerName,

                        ':customer_lastname' => $params->customerLastName,

                        ':customer_email' => $params->customerEmail,

                        ':customer_phone' => $params->customerPhone,

                        ':customer_gender' => $params->customerGender,

                        ':customer_birthdate' => $params->customerBirthdate,

                        ':customer_event_code' => $params->customerEventCode,
			':customer_event_date' => $params->customerEventDate,
                        ':customer_address_line1' => $params->customerAddressLine1,

                        ':customer_address_line2' => $params->customerAddressLine2,

                        ':customer_city' => $params->customerCity,

                        ':customer_state' => $params->customerState,

                        ':customer_zipcode' => $params->customerZipcode,

                        ':partner_fullname' => $params->partnerFullName,

                        ':partner_phone' => $params->partnerPhone,

                        ':partner_email' => $params->partnerEmail,

                        ':subtotal' => $params->subtotal,

                        ':tax' => $params->tax,

                        ':total' => $params->total,

                        ':discount_note' => $params->discountNote,

                        ':discount_amount' => $params->discountAmount,

                        ':order_status' => 'Order Received @ Designer Shop',

                        ':referral_code' => $params->referral,

                        ':coupon_code' => $params->couponCode,

                        ':coupon_amount_used' => $params->couponAmountUsed,

                        ':store' => $params->store,
			':fitting_date' => $params->fitting_date
                    );

        $pre->execute($values);



        // Get order id

        $sql = "SELECT MAX(order_id) as order_id from tblOrder where customer_email = :customerEmail";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

            ':customerEmail' => $params->customerEmail

        );

        $pre->execute( $values );

        $result = $pre->fetch();



        // Update coupon balance

        if( $params->couponCode != '' ) {

            $sql = "update tblCoupon set balance = balance - :coupon_amount_used, times_used = times_used + 1 where coupon_code = :coupon_code";

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                ':coupon_code' => $params->couponCode,

                ':coupon_amount_used' => $params->couponAmountUsed

            );

            $pre->execute( $values );

        }



        $order_id = null;

        if( count( $result ) > 0 )

           $order_id = $result['order_id'];



        // Save order details

        foreach( $params->items as $item ) {

            $sql = "INSERT INTO `tblOrderDetail` (`order_id`,

                                            `product_id`, 

                                            `product_name`, 

                                            `qty`,

                                            `price`,

                                            `total`,

                                            `image`,

                                            `is_couture`) VALUES (

                                            :order_id,

                                            :product_id,

                                            :product_name,

                                            :qty,

                                            :price,

                                            :total,

                                            :image,

                                            :is_couture)";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':order_id' => $order_id,

                            ':product_id' => $item->productId,

                            ':product_name' => $item->productName,

                            ':qty' => $item->qty,

                            ':price' => $item->price,

                            ':total' => intval( $item->qty ) * floatval( $item->price ),

                            ':image' => '<?xml version="1.0"?>' . $item->image,

                            ':is_couture' => $item->isCouture

                        );

            $pre->execute( $values );



            $sql = "SELECT max(item_id) item_id from tblOrderDetail where order_id = :order_id";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                ':order_id' => $order_id

            );

            $pre->execute( $values );

            $item_id = $pre->fetch()['item_id'];



            // Save item modifications

            foreach( $item->modifications as $mod ) {



                $sql = "INSERT INTO `tblOrderDetailCustomization` (`order_id`,

                                            `item_id`, 

                                            `customization_type`, 

                                            `category_id`,

                                            `category_name`,

                                            `option_id`,

                                            `option_name`,

                                            `option_image`,

                                            `couture_description`) VALUES (

                                            :order_id,

                                            :item_id,

                                            :customization_type,

                                            :category_id,

                                            :category_name,

                                            :option_id,

                                            :option_name,

                                            :option_image,

                                            :couture_description)";

                $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

                $values = array(

                                ':order_id' => $order_id,

                                ':item_id' => $item_id,

                                ':customization_type' => 'modification',

                                ':category_id' => $mod->catId,

                                ':category_name' => $mod->category,

                                ':option_id' => $mod->optId,

                                ':option_name' => $mod->option,

                                ':option_image' => $mod->optionImage,

                                ':couture_description' => $mod->description

                            );

                $pre->execute( $values );

            }



            // Save item fabrics 

            foreach( $item->fabrics as $fab ) {



                $sql = "INSERT INTO `tblOrderDetailCustomization` (`order_id`,

                                            `item_id`, 

                                            `customization_type`, 

                                            `category_id`,

                                            `category_name`,

                                            `option_id`,

                                            `option_name`,

                                            `option_image`) VALUES (

                                            :order_id,

                                            :item_id,

                                            :customization_type,

                                            :category_id,

                                            :category_name,

                                            :option_id,

                                            :option_name,

                                            :option_image)";

                $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

                $values = array(

                                ':order_id' => $order_id,

                                ':item_id' => $item_id,

                                ':customization_type' => 'fabric',

                                ':category_id' => $fab->fabric_option_id,

                                ':category_name' => $fab->fabric_option,

                                ':option_id' => $fab->fabric_id,

                                ':option_name' => $fab->fabric,

                                ':option_image' => $fab->fabric_image

                            );

                $pre->execute($values);

            }

        }



        // Save payment

        $sql = "INSERT INTO `tblOrderPayment` (`order_id`, 

                                        `payment_date`,

                                        `payment_amount`, 

                                        `memo`,

                                        `payment_method`,

                                        `payment_confirmation_number`) VALUES (

                                        :order_id,

                                        :payment_date,

                                        :payment_amount,

                                        'Initial deposit',

                                        :payment_method,

                                        :payment_confirmation_number)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $order_id,

                        ':payment_date' => date('Y-m-d h:i:s a', time()),

                        ':payment_amount' => floatval( str_replace( ',', '', $params->paymentAmount ) ),

                        ':payment_method' => $params->paymentMethod,

                        ':payment_confirmation_number' => $params->paymentConfirmationNumber

                    );

        $pre->execute($values);



        // Save order special charges

        foreach($params->specialCharges as $sc) {

            $sql = "INSERT INTO `tblOrderSpecialCharges` (`order_id`, 

                                            `special_charges_id`,

                                            `amount`) VALUES (

                                            :order_id,

                                            :special_charges_id,

                                            :amount)";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':order_id' => $order_id,

                            ':special_charges_id' => $sc->special_charges_id,

                            ':amount' => floatval( $sc->amount )

                        );

            $pre->execute($values);

        }



        // Save customer measurements

        $sql = "INSERT INTO `tblMeasurement` (`order_id`, 

                                        `height_feet`,

                                        `height_inch`, 

                                        `weight`,

                                        `neck`,

                                        `shoulder`,

                                        `sleeve_length`,

                                        `bicep`,

                                        `wrist`,

                                        `arms`,

                                        `chest`,

                                        `stomach`,

                                        `jacket_length_front`,

                                        `jacket_length_back`,

                                        `waist`,

                                        `hips`,

                                        `thigh`,

                                        `crotch`,

                                        `knee`,

                                        `leg_length`,

                                        `vest_length_front`,

                                        `vest_length_back`,

                                        `cuff`,

                                        `watch_hand`,

                                        `calf`,

                                        `sleeve_length_right`,

                                        `coat_sleeve_left`,

                                        `coat_sleeve_right`,

                                        `note`) VALUES (

                                        :order_id,

                                        :height_feet,

                                        :height_inch,

                                        :weight,

                                        :neck,

                                        :shoulder,

                                        :sleeve_length,

                                        :bicep,

                                        :wrist,

                                        :arms,

                                        :chest,

                                        :stomach,

                                        :jacket_length_front,

                                        :jacket_length_back,

                                        :waist,

                                        :hips,

                                        :thigh,

                                        :crotch,

                                        :knee,

                                        :leg_length,

                                        :vest_length_front,

                                        :vest_length_back,

                                        :cuff,

                                        :watch_hand,

                                        :calf,

                                        :sleeve_length_right,

                                        :coat_sleeve_left,

                                        :coat_sleeve_right,

                                        :note)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id'             => $order_id,

                        ':height_feet'          => $params->customerMeasurements->heightFeet,

                        ':height_inch'          => $params->customerMeasurements->heightInches,

                        ':weight'               => $params->customerMeasurements->weight,

                        ':neck'                 => $params->customerMeasurements->neck,

                        ':shoulder'             => $params->customerMeasurements->shoulder,

                        ':sleeve_length'        => $params->customerMeasurements->sleeveLengthLeft,

                        ':bicep'                => $params->customerMeasurements->bicep,

                        ':wrist'                => $params->customerMeasurements->wrist,

                        ':arms'                 => '',

                        ':chest'                => $params->customerMeasurements->chest,

                        ':stomach'              => $params->customerMeasurements->stomach,

                        ':jacket_length_front'  => $params->customerMeasurements->jacketLengthFront,

                        ':jacket_length_back'   => $params->customerMeasurements->jacketLengthBack,

                        ':waist'                => $params->customerMeasurements->waist,

                        ':hips'                 => $params->customerMeasurements->hips,

                        ':thigh'                => $params->customerMeasurements->thigh,

                        ':crotch'               => $params->customerMeasurements->crotch,

                        ':knee'                 => $params->customerMeasurements->knee,

                        ':leg_length'           => $params->customerMeasurements->legLength,

                        ':vest_length_front'    => $params->customerMeasurements->vestFront,

                        ':vest_length_back'     => $params->customerMeasurements->vestBack,

                        ':cuff'                 => $params->customerMeasurements->cuff,

                        ':watch_hand'           => $params->customerMeasurements->watchHand,

                        ':calf'                 => $params->customerMeasurements->calf,

                        ':sleeve_length_right'  => $params->customerMeasurements->sleeveLengthRight,

                        ':coat_sleeve_left'     => $params->customerMeasurements->coatSleeveLengthLeft,

                        ':coat_sleeve_right'    => $params->customerMeasurements->coatSleeveLengthRight,

                        ':note'                 => $params->customerMeasurements->measurementNotes

                    );

        $pre->execute($values);



        // Register order creation in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), 'system', 'Order created', 'creation', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $order_id,

                    );

        $pre->execute($values);



        // Register payment in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), 'system', :message, 'payment', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $order_id,

                        ':message' => 'A $' . $params->paymentAmount . ' USD payment was processed ' . ($params->paymentMethod == 'Stripe' ? 'on Stripe' : 'in store')

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $sql = $log->OrderCreation($order_id, $params->customerName . ' ' . $params->customerLastName, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // ------------------------------------------------------------------

        // Send mail notifications

        // LCS new order notification

        $sql = "select u.email, concat(u.first_name, ' ', u.last_name) full_name

                from tblUserPrivileges up, tblUser u

                where u.user_name = up.user_name and up.lcs = 1";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );

        $approvers = $pre->fetchall();



        foreach( $approvers as $lcs ) {

            sendEmail( 'lcs.neworder', $lcs['full_name'], $lcs['email'], $order_id );

        }



        // Customer new order notification

        sendEmail( 'client.neworder', $params->customerName . ' ' . $params->customerLastName, $params->customerEmail, $order_id );

        // ------------------------------------------------------------------



        // Return Order information

        $balance = floatval( $params->total ) - floatval( $params->discountAmount ) - floatval( $params->paymentAmount );

        return $response->withJson(

            array(

                'status' => 'Order Created', 

                'order' => array( 

                    'id' => $order_id, 

                    'status' => 'Received',

                    'subtotal' => $params->subtotal,

                    'tax' => $params->tax,

                    'total' => $params->total,

                    'payment_amount' => $params->paymentAmount,

                    'discount_amount' => $params->discountAmount,

                    'balance' => round($balance, 2) <= 0.01 ? 0.00 : round($balance, 2)

                )

            ), 200

        );

   }

   catch(\Exception $ex){

       return $response->withJson(array('error' => $ex->getMessage()),422);

   }

   

});



$app->post('/regpayment', function ($request, $response) {

    try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        // Validate request parameters

        //$errors = validateOrderParameters( $params );



        $con = $this->apidb;



        // Process payment

        if ( $params->paymentMethod == 'Stripe' ) {

            $curl = curl_init();

            curl_setopt_array($curl, array(

                CURLOPT_URL => "https://api.stripe.com/v1/charges",

                CURLOPT_RETURNTRANSFER => true,

                CURLOPT_ENCODING => "",

                CURLOPT_MAXREDIRS => 10,

                CURLOPT_TIMEOUT => 30,

                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,

                CURLOPT_CUSTOMREQUEST => "POST",

                CURLOPT_POSTFIELDS => "amount=" . ( round ($params->paymentAmount, 2) * 100 ) . 

                                    "&currency=usd&description=" . urlencode ( 'LFA - Design To Measure Order ' . $result['order_id'] ) . 

                                    "&source=" . $params->paymentToken,

                CURLOPT_HTTPHEADER => array(

                "Authorization: Bearer sk_live_Av8QgdpJTRmxz3V5LHpJkuxt",

                "Content-Type: application/x-www-form-urlencoded",

                "cache-control: no-cache"

                ),

            ));



            $stripe_response = curl_exec($curl);

            $err = curl_error($curl);



            curl_close($curl);



            if ($err) {

                return $response->withJson(array('status' => 'error', 'payment_error' => $err), 200);

            } 

        }

        // else, if it is the payment method is ATH / Credit Card at store, continue



        $sql = "INSERT INTO `tblOrderPayment` (`order_id`, 

                `payment_date`,

                `payment_amount`, 

                `memo`,

                `payment_method`,

                `payment_confirmation_number`) VALUES (

                :order_id,

                :payment_date,

                :payment_amount,

                :payment_message,

                :payment_method,

                :payment_confirmation_number)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

        ':order_id' => $params->orderId,

        ':payment_date' => date('Y-m-d h:i:s a', time()),

        ':payment_amount' => floatval( str_replace( ',', '', $params->paymentAmount ) ),

        ':payment_message' => 'Payment',

        ':payment_method' => $params->paymentMethod,

        ':payment_confirmation_number' => $params->paymentConfirmationNumber

        );

        $pre->execute($values);



        // Register order creation in order history

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), :user_name, :message, 'payment', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':user_name' => $token->getClaim('usr'),

                        ':message' => 'A $' . $params->paymentAmount . ' USD payment was processed ' . ($params->paymentMethod == 'Stripe' ? 'on Stripe' : 'in store')

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminRegisterPayment($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(

            array(

                'status' => 'Payment Processed', 

                'payment' => array( 

                    'id' => $params->orderId, 

                    'description' => 'Payment',

                    'date' => date('Y-m-d h:i:s a', time()),

                    'method' => $params->paymentMethod,

                    'confirmation' => $params->paymentConfirmationNumber,

                    'amount' => $params->paymentAmount

                )

            ), 200

        );       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

})->add($mw_authorize);



$app->post('/regpaymentnoauth', function ($request, $response) {

    try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        // Validate request parameters

        //$errors = validateOrderParameters( $params );



        $con = $this->apidb;



        // Process payment

        if ( $params->paymentMethod == 'Stripe' ) {

            $curl = curl_init();

            curl_setopt_array($curl, array(

                CURLOPT_URL => "https://api.stripe.com/v1/charges",

                CURLOPT_RETURNTRANSFER => true,

                CURLOPT_ENCODING => "",

                CURLOPT_MAXREDIRS => 10,

                CURLOPT_TIMEOUT => 30,

                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,

                CURLOPT_CUSTOMREQUEST => "POST",

                CURLOPT_POSTFIELDS => "amount=" . ( round ($params->paymentAmount, 2) * 100 ) . 

                                    "&currency=usd&description=" . urlencode ( 'LFA - Design To Measure Order ' . $result['order_id'] ) . 

                                    "&source=" . $params->paymentToken,

                CURLOPT_HTTPHEADER => array(

                "Authorization: Bearer sk_live_Av8QgdpJTRmxz3V5LHpJkuxt",

                "Content-Type: application/x-www-form-urlencoded",

                "cache-control: no-cache"

                ),

            ));



            $stripe_response = curl_exec($curl);

            $err = curl_error($curl);



            curl_close($curl);



            if ($err) {

                return $response->withJson(array('status' => 'error', 'payment_error' => $err), 200);

            } 

        }

        // else, if it is the payment method is ATH / Credit Card at store, continue



        $sql = "INSERT INTO `tblOrderPayment` (`order_id`, 

                `payment_date`,

                `payment_amount`, 

                `memo`,

                `payment_method`,

                `payment_confirmation_number`) VALUES (

                :order_id,

                :payment_date,

                :payment_amount,

                :payment_message,

                :payment_method,

                :payment_confirmation_number)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

        ':order_id' => $params->orderId,

        ':payment_date' => date('Y-m-d h:i:s a', time()),

        ':payment_amount' => floatval( str_replace( ',', '', $params->paymentAmount ) ),

        ':payment_message' => 'Payment',

        ':payment_method' => $params->paymentMethod,

        ':payment_confirmation_number' => $params->paymentConfirmationNumber

        );

        $pre->execute($values);



        // Register order creation in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), :user_name, :message, 'payment', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':user_name' => 'system',

                        ':message' => 'A $' . $params->paymentAmount . ' USD payment was processed ' . ($params->paymentMethod == 'Stripe' ? 'on Stripe' : 'in store')

                    );

        $pre->execute($values);



        return $response->withJson(

            array(

                'status' => 'Payment Processed', 

                'payment' => array( 

                    'id' => $params->orderId, 

                    'description' => 'Payment',

                    'date' => date('Y-m-d h:i:s a', time()),

                    'method' => $params->paymentMethod,

                    'confirmation' => $params->paymentConfirmationNumber,

                    'amount' => $params->paymentAmount

                )

            ), 200

        );       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

});



$app->get('/orderreceipt/{orderId}', function ($request, $response) {

    try {

        

        $con = $this->apidb;



        // Get order information 

        $sql = "SELECT order_id, order_date, rep_code, concat(customer_name, ' ', customer_lastname) customer_name, customer_email, customer_phone, customer_event_code, 

                       customer_address_line1, customer_address_line2, customer_city, customer_state, customer_zipcode, subtotal, tax, total, discount_note, discount_amount,

                       coupon_code, coupon_amount_used, o.store store_id, s.name store_name FROM tblOrder o, tblStore s WHERE o.store = s.id and order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $order = $pre->fetch();



        // Get order details

        $sql = "select od.item_id, product_id, product_name, qty, price, total, image, customization_type, category_id, category_name, option_id, option_name, option_image, od.is_couture, odc.couture_description

                from tblOrderDetail od, tblOrderDetailCustomization odc

                where od.item_id = odc.item_id and

                    od.order_id = :order_id

                order by od.item_id, customization_type desc";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $order_detail = $pre->fetchall();



        // Get order summary

        $sql = "select o.order_id, concat(o.customer_name, ' ', o.customer_lastname) customer_name, o.customer_email, o.customer_phone, o.subtotal, o.tax, o.total order_total, o.discount_note, o.discount_amount, o.status, o.tracking_number, o.carrier, o.pickup_date, o.coupon_code, o.coupon_amount_used, 

                        (select sum(payment_amount) from tblOrderPayment op where op.order_id = o.order_id) order_payment,

                        od.item_id, od.product_id, od.product_name, od.qty, od.price, od.total item_total, o.total - (select sum(payment_amount) from tblOrderPayment op where op.order_id = o.order_id) order_balance

                from tblOrder o, tblOrderDetail od

                where o.order_id = od.order_id and 

                    o.order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $order_summary = $pre->fetchall();



        // Get special charges details

        $sql = "SELECT s.name, os.amount FROM tblOrderSpecialCharges os, tblSpecialCharges s WHERE os.special_charges_id = s.special_charges_id and order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $order_special_charges = $pre->fetchall();



        // Register action in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), 'system', 'Receipt generated', 'receipt', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $request->getAttribute('orderId'),

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $sql = $log->AdminExportPdf('system', $request->getAttribute('orderId'), 'Someone generated the order receipt.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> array( 'order' => $order, 'order_detail' => $order_detail, 'order_summary' => $order_summary, 'special_charges' => $order_special_charges )), 200);

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

});



$app->get('/forder/{orderId}', function ($request, $response) {

    try {

        

        $con = $this->apidb;



        // Get order information 

        $sql = "SELECT order_id, order_date, rep_code, concat(customer_name, ' ', customer_lastname) customer_name, customer_email, customer_phone, customer_gender, customer_event_code, 

                       customer_address_line1, customer_address_line2, customer_city, customer_state, customer_zipcode, subtotal, tax, total, discount_note, discount_amount,

                       coupon_code, coupon_amount_used, o.store store_id, s.name store_name FROM tblOrder o, tblStore s WHERE o.store = s.id and order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $order = $pre->fetch();



        // Get order details

        $sql = "select od.item_id, product_id, product_name, qty, price, total, image, customization_type, category_id, category_name, option_id, option_name, option_image, od.is_couture, odc.couture_description

                from tblOrderDetail od, tblOrderDetailCustomization odc

                where od.item_id = odc.item_id and

                    od.order_id = :order_id

                order by od.item_id, customization_type desc";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $order_detail = $pre->fetchall();



        // Get Measurements

        $sql = "select *

                from tblMeasurement m

                where m.order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $measurements = $pre->fetch();



        // Register action in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), 'system', 'Factory order generated', 'factory order', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $request->getAttribute('orderId'),

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminExportPdf($token->getClaim('usr'), $request->getAttribute('orderId'), 'The user generated the Factory Order PDF.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> array( 'order' => $order, 'order_detail' => $order_detail, 'measurements' => $measurements )), 200);

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

})->add($mw_authorize);



$app->get('/coupon/{coupoCode}', function($request, $response) {

    try{

        

        $con = $this->apidb;

        $sql = "select * from tblCoupon where coupon_code = :coupon_code";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

            ':coupon_code' => $request->getAttribute('coupoCode')

        );

        $pre->execute( $values );

        $result = $pre->fetch();



        if(!$result)

            return $response->withJson(array('status' => 'false', 'message' => 'Coupon not valid'));



        return $response->withJson(array('status' => 'true','data'=> $result), 200);       

    }

    catch(\Exception $ex) {

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

});



$app->get('/admin/orderlist', function ($request, $response) {

    try{

        

        $con = $this->apidb;

        $sql = "select o.order_id orderId, o.order_date orderDate, concat(o.customer_name, ' ', o.customer_lastname) customerName, o.customer_phone customerPhone, o.customer_event_date customerEventDate, o.fitting_date customerFittingDate, o.total, o.status orderStatus, o.total - (select sum(payment_amount) from tblOrderPayment op where op.order_id = o.order_id) balance

                from tblOrder o

                order by o.order_date desc";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( );

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminOrderList($token->getClaim('usr'), $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        if( $result ) {

            return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token')), 200);

        } else {

            return $response->withJson(array('status' => 'No orders has been made.', 'token' => $request->getAttribute('token')), 422);

        }

       

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

})->add($mw_authorize);



$app->get('/admin/orderdetails/{orderId}', function ($request, $response) {

    try {

        

        $con = $this->apidb;

        $sql = "select o.order_id orderId, o.order_date orderDate, o.customer_name customerFirstName, o.customer_lastname customerLastName, concat(o.customer_name, ' ', o.customer_lastname) customerName, o.customer_gender customerGender, o.customer_birthdate customerBirthdate, o.customer_event_code eventCode, o.customer_event_date eventDate,

                       o.customer_address_line1 customerAddressLine1, o.customer_address_line2 customerAddressLine2, o.customer_city customerCity, o.customer_state customerState, o.customer_zipcode customerZipcode, o.customer_phone customerPhone, 

                       o.customer_email customerEmail, o.total, o.status orderStatus, o.total - (select sum(payment_amount) from tblOrderPayment op where op.order_id = o.order_id) balance,

                       o.subtotal, o.tax, o.discount_note discountNote, o.discount_amount discount, (select rep_name from tblRepresentative r where type = 'Stylist' and r.rep_code = o.rep_code) rep, o.carrier, o.tracking_number trackingNumber, o.pickup_date pickupDate, o.partner_fullname partnerFullname, 

                       o.partner_email partnerEmail, o.partner_phone partnerPhone, o.referral_code referralCode, o.coupon_code couponCode, o.coupon_amount_used couponAmountUsed,
		       (select name from tblStore where id = o.store) store, o.fitting_date fittingDate

                from tblOrder o

                where order_id = :order_id

                order by o.order_date desc";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $result = $pre->fetch();



        // Get order details

        $sql = "select od.item_id itemId, product_id productId, product_name productName, qty, price, total, is_couture isCouture, customization_type customizationType, category_id categoryId, category_name categoryName, 

                       option_id optionId, option_name optionName, option_image optionImage, couture_description coutureDescription

                from tblOrderDetail od, tblOrderDetailCustomization odc

                where od.item_id = odc.item_id and

                    od.order_id = :order_id

                order by od.item_id, customization_type desc";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $items = $pre->fetchall();



        // Get order notes

        $sql = "select order_id orderId, cast(note_date as date) noteDate, TIME_FORMAT(cast(note_date as time), '%H:%i %p') noteTime, concat(first_name, ' ', last_name) author, message

                from tblOrderNote n, tblUser u

                where n.author = u.user_name and order_id = :order_id

                order by n.note_date";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $notes = $pre->fetchall();



        // Get order payments

        $sql = "select DATE_FORMAT(payment_date, '%Y-%m-%d @ %H:%i %p') paymentDate, payment_amount paymentAmount, memo, payment_method paymentMethod, payment_confirmation_number paymentConfirmationNumber

                from tblOrderPayment

                where order_id = :order_id

                order by payment_date desc";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $payments = $pre->fetchall();



        // Get customer measures

        $sql = "select concat(height_feet, '\' ', height_inch, '\'\'') height, height_feet, height_inch, weight, neck, shoulder, sleeve_length sleeve_length_left, sleeve_length_right, bicep, wrist, chest, 

                       stomach, jacket_length_front, jacket_length_back, waist, hips, thigh, crotch, knee, leg_length, vest_length_front, 

                       vest_length_back, cuff, watch_hand, calf, coat_sleeve_left, coat_sleeve_right, sleeve_length_right, note

                from tblMeasurement where order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $measures = $pre->fetch();



        // Get order history

        $sql = "select cast(action_date as date) date, TIME_FORMAT(cast(action_date as time), '%H:%i %p') time, 

                    case when user_name = 'system' then '' else user_name end user, action, action_type type, new_status status, 

                    case when action_type = 'payment' then 'blue' else 'grey' end color, 

                    case when new_status = 'Delivered to Customer' then 'green'

                         when new_status = 'Approved by Designer' then 'blue-grey' else 'orange' end statusColor 

                from tblOrderHistory 

                where order_id = :order_id

                order by action_date";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $history = $pre->fetchall();



        // Get order special charges

        $sql = "select sc.special_charges_id specialChargesId, sc.name, sc.description, sc.amount, sc.active

                from tblOrderSpecialCharges osc, tblSpecialCharges sc

                where sc.special_charges_id = osc.special_charges_id and osc.order_id = :order_id";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $request->getAttribute('orderId') );

        $pre->execute( $values );

        $special_charges = $pre->fetchall();



        $sql = "select status from tblOrderStatus order by status_order";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( );

        $pre->execute( $values );

        $status = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminOrderDetail($token->getClaim('usr'), $request->getAttribute('orderId'), $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        if( $result && $items ) {

            return $response->withJson(array('status' => 'true','data'=> array( 'order' => $result, 'items' => $items, 'notes' => $notes, 'payments' => $payments, 'measurements' => $measures, 'history' => $history, 'special_charges' => $special_charges, 'status' => $status ), 'token' => $request->getAttribute('token') ), 200);

        } else {

            return $response->withJson(array('status' => 'No order found.', 'token' => $request->getAttribute('token')), 422);

        }

    }

    catch(\Exception $ex) {

        return $response->withJson(array('error' => $ex->getMessage()),422);

    }

})->add($mw_authorize);



$app->post('/admin/orderstatus', function ($request, $response) {

    try{

        $token = (new Parser())->parse($request->getAttribute('token'));

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;

        // Change order status

        $sql = "update `tblOrder` set `status` = :order_status where order_id = :order_id";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':order_status' => $params->status

                    );

        $pre->execute($values);



        // Register action in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), :user, 'Status changed', 'status', :order_status)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':user' => $token->getClaim('usr'),

                        ':order_status' => $params->status

                    );

        $pre->execute($values);



        $status_color = 'orange';

        switch( $params->status ) {

            case 'Delivered to Customer':

                $status_color = 'green';

                break;

            case 'Approved by Designer':

                $status_color = 'blue-grey';

                break;

        }



        // log action

        $log = new LogQueries();

        $sql = $log->AdminChangeOrderStatus($token->getClaim('usr'), $params->orderId, $params->status, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Send mail notification

        // Send admin notification when the designer has approved the order

        if( $params->status == 'Approved by Designer' ) {

            $sql = "select concat(u.first_name, ' ', u.last_name) full_name, u.email

                    from tblUser u, tblUserPrivileges up

                    where u.user_name = up.user_name and up.manage_order = 1 and up.lcs = 0";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $pre->execute( );

            $admins = $pre->fetchall();

            foreach( $admins as $admin ) {
                sendEmail( 'admin.orderapproved', $admin['full_name'], $admin['email'], $params->orderId );
            }
        }

        

        // Send customer order update

        $sql = "select concat(customer_name, ' ', customer_lastname) customer_full_name, customer_email 

                from tblOrder

                where order_id = :order_id";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                    );

        $pre->execute($values);

        $customer = $pre->fetch();

        sendEmail( 'client.statuschange', $customer['customer_full_name'], $customer['customer_email'], $params->orderId, $params->status );



        // Return Order information

        return $response->withJson(

            array( 'status' => 'true','data'=> array( 

                    'status' => $params->status, 

                    'user' => $params->user,

                    'date' => date('Y-m-d', time()),

                    'time' => date('h:i A', time()),

                    'action' => 'Status changed',

                    'color' => 'grey',

                    'statusColor' => $status_color, 

                    'token' => $request->getAttribute('token')

                )

            ), 200

        );

       

   }

   catch(\Exception $ex) {

       return $response->withJson(array('error' => $ex->getMessage()), 422);

   }

})->add($mw_authorize);

$app->post('/admin/customeremail', function ($request, $response) {
    try{
        $token = (new Parser())->parse($request->getAttribute('token'));
        // Get all the request values
        // $params = getOrderParams( $request );
        $params = json_decode( $request->getParsedBody()['form'] )->form;

        $con = $this->apidb;

        // Register action in order history
        $sql = "insert into `tblOrderHistory` values (:order_id, now(), :user, 'Message sent to customer', 'notification', null)";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
                        ':order_id' => $params->orderId,
                        ':user' => $token->getClaim('usr')
                    );
        $pre->execute($values);

        // log action
        $log = new LogQueries();
        $sql = $log->AdminMessageToCustomer($token->getClaim('usr'), $params->orderId, $params->message, $request->getAttribute('ip_address'), '', '');
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( );
        
        // Send customer order update
        $sql = "select concat(customer_name, ' ', customer_lastname) customer_full_name, customer_email 
                from tblOrder
                where order_id = :order_id";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
            ':order_id' => $params->orderId,
        );
        $pre->execute($values);
        $customer = $pre->fetch();
        sendCustomerEmail( $customer['customer_email'], $customer['customer_full_name'],$params->subject, $params->message );

        // Return Order information
        return $response->withJson(
            array( 'status' => true, 'error' => false, 'message' => 'SUCCESS', 'token' => $request->getAttribute('token') )
            , 200
        );
       
   }
   catch(\Exception $ex) {
       return $response->withJson(array('error' => $ex->getMessage()), 422);
   }
})->add($mw_authorize);

$app->post('/admin/ordernote', function ($request, $response) {

    try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        $token = (new Parser())->parse($request->getAttribute('token'));

        $note = registerNote( $params, $con, $token->getClaim('usr') );



        // log action

        $log = new LogQueries();

        $sql = $log->AdminAddNote($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return Order information

        return $response->withJson( array('status' => 'true','data'=> $note, 'token' => $request->getAttribute('token') ), 200 );

       

   }

   catch(\Exception $ex){

       return $response->withJson(array('error' => $ex->getMessage()),422);

   }

})->add($mw_authorize);



$app->post('/admin/ordershipping', function ($request, $response) {

    try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;

        // Change order status

        $sql = "update `tblOrder` set tracking_number = :tracking_number, carrier = :carrier where order_id = :order_id";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':carrier' => $params->carrier,

                        ':tracking_number' => $params->trackingNumber

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminChangeOrderShipping($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return Order information

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );

       

   }

   catch(\Exception $ex){

       return $response->withJson(array('error' => $ex->getMessage()),422);

   }

})->add($mw_authorize);



$app->post('/admin/ordermeasurements', function ($request, $response) {

    try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;

        // Change order status

        $sql = "update `tblMeasurement` set 

                    height_feet = :heightFeet,

                    height_inch = :heightInches,

                    weight = :weight,

                    neck = :neck,

                    shoulder = :shoulder,

                    sleeve_length = :sleeveLengthLeft,

                    bicep = :bicep,

                    wrist = :wrist,

                    chest = :chest,

                    stomach = :stomach,

                    jacket_length_front = :jacketLengthFront,

                    jacket_length_back = :jacketLengthBack,

                    waist = :waist,

                    hips = :hips,

                    thigh = :thigh,

                    crotch = :crotch,

                    knee = :knee,

                    leg_length = :legLength,

                    vest_length_front = :vestLengthFront,

                    vest_length_back = :vestLengthBack,

                    cuff = :cuff,

                    watch_hand = :watchHand,

                    calf = :calf,

                    sleeve_length_right = :sleeveLengthRight,

                    coat_sleeve_left = :coatSleeveLeft,

                    coat_sleeve_right = :coatSleeveright

                where order_id = :order_id";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':heightFeet' => $params->heightFeet,

                        ':heightInches' => $params->heightInches,

                        ':weight' => $params->weight,

                        ':neck' => $params->neck,

                        ':shoulder' => $params->shoulder,

                        ':sleeveLengthLeft' => $params->sleeveLengthLeft,

                        ':bicep' => $params->bicep,

                        ':wrist' => $params->wrist,

                        ':chest' => $params->chest,

                        ':stomach' => $params->stomach,

                        ':jacketLengthFront' => $params->jacketLengthFront,

                        ':jacketLengthBack' => $params->jacketLengthBack,

                        ':waist' => $params->waist,

                        ':hips' => $params->hips,

                        ':thigh' => $params->thigh,

                        ':crotch' => $params->crotch,

                        ':knee' => $params->knee,

                        ':legLength' => $params->legLength,

                        ':vestLengthFront' => $params->vestLengthFront,

                        ':vestLengthBack' => $params->vestLengthBack,

                        ':cuff' => $params->cuff,

                        ':watchHand' => $params->watchHand,

                        ':calf' => $params->calf,

                        ':sleeveLengthRight' => $params->sleeveLengthRight,

                        ':coatSleeveLeft' => $params->coatSleeveLengthLeft,

                        ':coatSleeveright' => $params->coatsleeveLengthRight

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminUpdateOrderCustomer($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return Order information

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );

   }

   catch(\Exception $ex) {

       return $response->withJson(array('error' => $ex->getMessage()),422);

   }

})->add($mw_authorize);

$app->post('/admin/ordercustomerinfo', function ($request, $response) {
    try{
        // Get all the request values
        // $params = getOrderParams( $request );
        $params = json_decode( $request->getParsedBody()['form'] )->form;

        $con = $this->apidb;
        // Change order status
        $sql = "update `tblOrder` set 
                    customer_name = :customer_name,
                    customer_lastname = :customer_lastname,
                    customer_email = :customer_email,
                    customer_phone = :customer_phone,
                    customer_gender = :customer_gender,
                    customer_birthdate = :customer_birthdate,
                    customer_address_line1 = :customer_address_line1,
                    customer_address_line2 = :customer_address_line2,
                    customer_city = :customer_city,
                    customer_state = :customer_state,
                    customer_zipcode = :customer_zipcode,
                    customer_event_code = :customer_event_code,
                    customer_event_date = :customer_event_date,
                    fitting_date = :fitting_date
                where order_id = :order_id";
        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $values = array(
                        ':order_id' => $params->orderId,
                        ':customer_name' => $params->firstName,
                        ':customer_lastname' => $params->lastName,
                        ':customer_email' => $params->email,
                        ':customer_phone' => $params->phone,
                        ':customer_gender' => $params->gender,
                        ':customer_birthdate' => $params->birthdate,
                        ':customer_address_line1' => $params->addressLine1,
                        ':customer_address_line2' => $params->addressLine2,
                        ':customer_city' => $params->addressCity,
                        ':customer_state' => $params->addressState,
                        ':customer_zipcode' => $params->addressZipcode,
                        ':customer_event_code' => $params->eventCode,
                        ':customer_event_date' => $params->eventDate,
                        ':fitting_date' => $params->fittingDate
                    );
        $pre->execute($values);

        // log action
        $log = new LogQueries();
        $token = (new Parser())->parse($request->getAttribute('token'));
        $sql = $log->AdminUpdateMeasurements($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '');
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( );

        // Return Order information
        return $response->withJson(
            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200
        );
   }
   catch(\Exception $ex) {
       return $response->withJson(array('error' => $ex->getMessage()),422);
   }
})->add($mw_authorize);

$app->post('/admin/updateattribute', function ($request, $response) {

    try{

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        // Make he update

        $con = $this->apidb;

        $sql = "update `tblOrderDetailCustomization` set option_id = :option_id, option_name = :option_name, option_image = :option_image where order_id = :order_id and customization_type = :type and item_id = :item_id and category_id = :category_id";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':option_id' => $params->toValue->optionId,

                        ':option_name' => $params->toValue->optionName,

                        ':option_image' => $params->toValue->imageUrl,

                        ':order_id' => $params->orderId,

                        ':type' => $params->type,

                        ':item_id' => $params->itemId,

                        ':category_id' => $params->categoryId

                    );

        $pre->execute($values);

        

        // Make log entry



        // Register note

        $token = (new Parser())->parse($request->getAttribute('token'));

        $noteParams = (object)array( 'orderId' => $params->orderId, 'user' => $params->user, 'message' => '[' . $params->type . ' update] ' . $params->note );

        $note = registerNote( $noteParams, $con, $token->getClaim('usr') );



        // Regiter action in order history

        $sql = "insert into `tblOrderHistory` values (:order_id, now(), :user, :action, 'update', null)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':order_id' => $params->orderId,

                        ':user' => $params->user,

                        ':action' => 'Updated item ' . $params->itemName . ' [' . $params->itemId . '], change the ' . 

                                      $params->categoryName . ' ' . ($params->type == 'modifications' ? 'attribute' : 'fabric') . 

                                      ' from ' . $params->fromValue->optionNameFormatted . ' to ' . 

                                      $params->toValue->optionNameFormatted . '.',

                    );

        $pre->execute($values);



        $sql = "select cast(action_date as date) date, TIME_FORMAT(cast(action_date as time), '%H:%i %p') time, 

                    case when user_name = 'system' then '' else user_name end user, action, action_type type, new_status status, 

                    case when action_type = 'payment' then 'blue' else 'grey' end color, 

                    case when new_status = 'Completed' then 'green'

                         when new_status = 'Cancelled' then 'blue-grey'

                         when new_status = 'Refunded' then 'grey' else 'orange' end statusColor 

                from tblOrderHistory 

                where order_id = :order_id

                order by action_date desc

                limit 1";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array( ':order_id' => $params->orderId );

        $pre->execute( $values );

        $history = $pre->fetch();



        // log action

        $log = new LogQueries();

        if ( $params->type == 'modification' )

            $sql = $log->AdminUpdateModifications($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '', $params->itemId, $params->itemName, $params->categoryName, $params->toValue->optionNameFormatted);

        else 

            $sql = $log->AdminUpdateFabric($token->getClaim('usr'), $params->orderId, $request->getAttribute('ip_address'), '', '', $params->itemId, $params->itemName, $params->categoryName, $params->toValue->optionNameFormatted);

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return Order information

        return $response->withJson(

            array('status' => 'true', 'data' => array( 'note' => $note, 'historyItem' => $history ), 'token' => $request->getAttribute('token') ), 200

        );

       

   }

   catch(\Exception $ex){

       return $response->withJson(array('error' => $ex->getMessage()),422);

   }

})->add($mw_authorize);



$app->get('/admin/rpt.journal/{date}', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select o.order_id orderId, cast(o.order_date as date) orderDate, concat(o.customer_name, ' ', o.customer_lastname) customerName, o.customer_gender customerGender, o.customer_birthdate customerBirthdate, o.customer_event_code eventCode, 

                        o.customer_address_line1 customerAddressLine1, o.customer_address_line2 customerAddressLine2, o.customer_city customerCity, o.customer_state customerState, o.customer_zipcode customerZipcode, o.customer_phone customerPhone, 

                        o.customer_email customerEmail, o.total, o.status orderStatus, o.total - (select sum(payment_amount) from tblOrderPayment op where op.order_id = o.order_id) balance,

                        o.subtotal, o.tax, o.discount_note discountNote, o.discount_amount discount, o.rep_code rep, (select rep_name from tblRepresentative where type = 'stylist' and rep_code = o.rep_code) stylist, o.carrier, o.tracking_number trackingNumber, o.pickup_date pickupDate, o.partner_fullname partnerFullname, 

                        o.partner_email partnerEmail, o.partner_phone partnerPhone, o.referral_code referralCode, (select rep_name from tblRepresentative where type = 'referral' and rep_code = o.referral_code) referral, o.coupon_code couponCode, o.coupon_amount_used couponAmountUsed, p.payment_method paymentMethod, 

                        p.payment_confirmation_number paymentConfirmationNumber

                from tblOrder o, tblOrderPayment p

                where o.order_id = p.order_id and cast(p.payment_date as date) ";

        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";

        $sql .= "order by p.payment_date asc";



        $values = array( ':date_from' => $date[0] );

        if( count($date) > 1 )

            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );



        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminReport($token->getClaim('usr'), 'Journal', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/admin/rpt.cuadre/{date}', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select p.payment_method paymentMethod, sum(o.subtotal) subtotal, sum(o.tax) tax, sum(o.total) total

                from tblOrder o, tblOrderPayment p

                where o.order_id = p.order_id and cast(p.payment_date as date) ";

        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";

        $sql .= "group by p.payment_method order by p.payment_method asc";



        $values = array( ':date_from' => $date[0] );

        if( count($date) > 1 )

            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );



        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminReport($token->getClaim('usr'), 'Cuadre', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/rpt.salesbyitem/{date}', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select od.product_id, od.product_name itemName, sum(od.qty) qty, sum(od.total) total 

                from tblOrderDetail od, tblOrder o

                where o.order_id = od.order_id and cast(o.order_date as date) ";

        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";

        $sql .= "group by od.product_id, od.product_name order by sum(od.total) desc";



        $values = array( ':date_from' => $date[0] );

        if( count($date) > 1 )

            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );



        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminReport($token->getClaim('usr'), 'Sales by Item', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/rpt.salesbystore/{date}', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select  o.store, s.name storeName, count(1) orderQty, sum((select count(1) from tblOrderDetail od where od.order_id = o.order_id)) itemQty, sum(total) total

                from tblOrder o, tblStore s 

                where o.store = s.id and cast(o.order_date as date) ";

        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";

        $sql .= "group by o.store order by sum(o.total) desc";



        $values = array( ':date_from' => $date[0] );

        if( count($date) > 1 )

            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );



        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminReport($token->getClaim('usr'), 'Sales by Store', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/rpt.salesbyrep/{date}', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select o.rep_code repCode, r.rep_name repName, count(distinct o.order_id) orderQty, sum(od.qty) qty, sum(od.total) total 

                from tblOrderDetail od, tblOrder o, tblRepresentative r

                where o.order_id = od.order_id

                      and r.rep_code = o.rep_code and cast(o.order_date as date) ";

        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";

        $sql .= "group by o.rep_code, r.rep_name order by sum(od.total) desc";



        $values = array( ':date_from' => $date[0] );

        if( count($date) > 1 )

            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );



        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminReport($token->getClaim('usr'), 'Sales by Stylist', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/rpt.salesbymonth/{date}', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select DATE_FORMAT(o.order_date,'%M - %Y') orderDate, count(distinct o.order_id) orderQty, sum(od.qty) qty, sum(od.total) total 

                from tblOrderDetail od, tblOrder o

                where o.order_id = od.order_id

                    and cast(o.order_date as date) ";

        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";

        $sql .= "group by DATE_FORMAT(o.order_date,'%M - %Y') order by sum(od.total) desc";



        $values = array( ':date_from' => $date[0] );

        if( count($date) > 1 )

            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );



        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminReport($token->getClaim('usr'), 'Sales by Month', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);

$app->get('/rpt.ordersbystatus/{date}', function($request, $response) {
    try {
        $date = explode(';', $request->getAttribute('date'));
        $con = $this->apidb;
        $sql = "select o.status, count(distinct o.order_id) orderQty, sum(o.total) total 
                from tblOrder o
                where cast(o.order_date as date) ";
        $sql .= count($date) > 1 ? "between :date_from and :date_to " : "= :date_from ";
        $sql .= "group by o.status order by sum(o.total) desc";

        $values = array( ':date_from' => $date[0] );
        if( count($date) > 1 )
            $values = array( ':date_from' => $date[0], ':date_to' => $date[1] );

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( $values );
        $result = $pre->fetchall();

        // log action
        $log = new LogQueries();
        $token = (new Parser())->parse($request->getAttribute('token'));
        $sql = $log->AdminReport($token->getClaim('usr'), 'Orders by Status', $request->getAttribute('ip_address'), '', '');
        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));
        $pre->execute( );

        return $response->withJson(array('status' => 'true','data'=> $result, 'token' => $request->getAttribute('token') ), 200);
    }
    catch(\Exception $ex){
        return $response->withJson(array('error' => $ex->getMessage()), 422);
    }
})->add($mw_authorize);

$app->get('/admin/dashboard', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "CALL spDashboard";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        $result = array();

        $rows = $pre->fetchAll();

        array_push( $result, $rows[0] );



        $pre->nextRowset();



        $rows = $pre->fetchAll();

        array_push( $result, $rows );



        $pre->nextRowset();



        $rows = $pre->fetchAll();

        array_push( $result, $rows[0] );



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminDashboard($token->getClaim('usr'), $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => true,'data'=> $result, 'token' => $request->getAttribute('token') ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/newuser', function($request, $response) {

    try {



        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        // If we are editing an existing user

        if( $params->isEdit ) {



            $sql = "update tblUser set first_name = :first_name, last_name = :last_name, email = :email where user_name = :user_name";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':first_name' => $params->firstName,

                            ':last_name' => $params->lastName,

                            ':email' => $params->email,

                            ':user_name' => $params->userName

                        );

            $pre->execute($values);



            $sql = "update tblUserPrivileges set manage_order = :manage_order, manage_user = :manage_user, manage_stylist = :manage_stylist, 

                        manage_referrals = :manage_referrals, manage_coupon = :manage_coupon, view_log = :view_log, lcs = :lcs, view_reports = :view_reports 

                        where user_name = :user_name";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':manage_order' => $params->manageOrder,

                            ':manage_user' => $params->manageUser,

                            ':manage_stylist' => $params->manageStylist,

                            ':manage_referrals' => $params->manageReferral,

                            ':manage_coupon' => $params->manageCoupon,

                            ':view_log' => $params->viewLog,

                            ':lcs' => $params->lcs,

                            ':view_reports' => $params->reports,

                            ':user_name' => $params->userName

                        );

            $pre->execute($values);



            // log action

            $log = new LogQueries();

            $token = (new Parser())->parse($request->getAttribute('token'));

            $sql = $log->AdminAction($token->getClaim('usr'), 'The user update the user ' . $params->userName . ' configuration.', $request->getAttribute('ip_address'), '', '');

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $pre->execute( );



            return $response->withJson(array('status' => 'true', 'token' => $request->getAttribute('token') ), 200);

        }



        // Check if the email already exist

        $sql = "select email from tblUser where email = :email";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':email' => $params->email

                    );

        $pre->execute($values);

        $result = $pre->fetchAll();



        if ( count($result) > 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'There is already a user with the given email.' ), 200 );



        // Check if the username already exist

        $exist = false;

        $i = 0;

        $un = '';

        do {

            $un = $params->userName;

            

            if($i > 0) 

                $un = $params->userName . $i;



            $sql = "select user_name from tblUser where user_name = :user_name";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':user_name' => $un

                        );

            $pre->execute($values);

            $result = $pre->fetchAll();



            if ( count($result) > 0 )

                $exist = true;

            else

                $exist = false;

            

            $i++;

        }

        while( $exist );



        // Save user information

        $pwd = bin2hex(openssl_random_pseudo_bytes(4));

        $sql = "insert into tblUser (

                            user_name,

                            password,

                            first_name,

                            last_name,

                            creation_date,

                            email

                        ) values (

                            :user_name,

                            :password,

                            :first_name,

                            :last_name,

                            cast(NOW() as date),

                            :email)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':user_name' => $un,

                        ':password' => password_hash ( $pwd, PASSWORD_DEFAULT ),

                        ':first_name' => $params->firstName,

                        ':last_name' => $params->lastName,

                        ':email' => $params->email

                    );

        $pre->execute($values);



        // Save user privileges

        $sql = "insert into tblUserPrivileges (

                            user_name, 

                            manage_order, 

                            manage_user, 

                            manage_stylist, 

                            manage_referrals, 

                            view_log, 

                            lcs,

                            view_reports

                        ) values (

                            :user_name,

                            :manage_order,

                            :manage_user,

                            :manage_stylist,

                            :manage_referrals,

                            :view_log,

                            :lcs,

                            :view_reports)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':user_name' => $un,

                        ':manage_order' => $params->manageOrder,

                        ':manage_user' => $params->manageUser,

                        ':manage_stylist' => $params->manageStylist,

                        ':manage_referrals' => $params->manageReferral,

                        ':view_log' => $params->viewLog,

                        ':lcs' => $params->lcs,

                        ':view_reports' => $params->reports

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user created the user ' . $un . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'userName' => $un, 'token' => $request->getAttribute('token') ), 200

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/newrep', function($request, $response) {

    try {



        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        // If we are editing an existing user

        if( $params->isEdit ) {



            $sql = "update tblRepresentative set rep_name = :rep_name, active = :active where rep_code = :rep_code";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':rep_name' => $params->repName,

                            ':active' => $params->active ? 1 : 0,

                            ':rep_code' => $params->repCode

                        );

            $pre->execute($values);



            // log action

            $log = new LogQueries();

            $token = (new Parser())->parse($request->getAttribute('token'));

            $sql = $log->AdminAction($token->getClaim('usr'), 'The user update the representative ' . $params->repName . ' configuration.', $request->getAttribute('ip_address'), '', '');

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $pre->execute( );



            return $response->withJson(array('status' => 'true', 'token' => $request->getAttribute('token') ), 200);

        }



        // Check if the email already exist

        $sql = "select rep_code from tblRepresentative where rep_code = :rep_code and type = :type";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':rep_code' => $params->repCode,

                        ':type' => $params->type

                    );

        $pre->execute($values);

        $result = $pre->fetchAll();



        if ( count($result) > 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'There is already a representative with the given code.' ), 200 );



        // Save user information

        $sql = "insert into tblRepresentative (

                            rep_code,

                            rep_name,

                            active,

                            type

                        ) values (

                            :rep_code,

                            :rep_name,

                            :active,

                            :type)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':rep_code' => $params->repCode,

                        ':rep_name' => $params->repName,

                        ':active' => $params->active ? 1 : 0,

                        ':type' => $params->type

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user created the representative ' . $params->repName . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/newcoupon', function($request, $response) {

    try {



        // Get all the request values

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        // If we are editing an existing coupon

        if( $params->isEdit ) {



            $sql = "update tblCoupon set valid_from = :valid_from, valid_to = :valid_to, active = :active where coupon_code = :coupon_code";

            $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $values = array(

                            ':valid_from' => $params->validFrom,

                            ':valid_to' => $params->validTo,

                            ':active' => $params->active ? 1 : 0,

                            ':coupon_code' => $params->couponCode

                        );

            $pre->execute($values);



            // log action

            $log = new LogQueries();

            $token = (new Parser())->parse($request->getAttribute('token'));

            $sql = $log->AdminAction($token->getClaim('usr'), 'The user edited the coupon code ' . $params->couponCode . '.', $request->getAttribute('ip_address'), '', '');

            $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

            $pre->execute( );



            return $response->withJson(array('status' => 'true', 'token' => $request->getAttribute('token') ), 200);

        }



        // Check if the coupon code already exist

        $sql = "select coupon_code from tblCoupon where coupon_code = :coupon_code";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':coupon_code' => $params->couponCode

                    );

        $pre->execute($values);

        $result = $pre->fetchAll();



        if ( count($result) > 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'There is already a coupon with the given code.' ), 200 );



        // Save user information

        $sql = "insert into tblCoupon (

                            coupon_code,

                            created_date,

                            created_by,

                            valid_from,

                            valid_to,

                            discount_amount,

                            discount_type,

                            active,

                            balance,

                            times_used

                        ) values (

                            :coupon_code,

                            NOW(),

                            :created_by,

                            :valid_from,

                            :valid_to,

                            :discount_amount,

                            :discount_type,

                            :active,

                            :balance,

                            0)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':coupon_code' => $params->couponCode,

                        ':created_by' => $params->user,

                        ':valid_from' => $params->validFrom,

                        ':valid_to' => $params->validTo,

                        ':discount_amount' => $params->amount,

                        ':discount_type' => $params->type,

                        ':active' => $params->active ? 1 : 0,

                        ':balance' => $params->amount

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user created the coupon code ' . $params->couponCode . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );

    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/toggleuser', function($request, $response) {

    try {



        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        $sql = "update tblUser set active = :active where user_name = :user_name";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':active' => $params->active ? 1 : 0,

                        ':user_name' => $params->userName

                    );

        $pre->execute($values);



        // log action

        $act = $params->active ? 'activated' : 'deactivated';

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user ' . $act . ' the user ' . $params->userName . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );

    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/togglecoupon', function($request, $response) {

    try {



        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        $sql = "update tblCoupon set active = :active where coupon_code = :coupon_code";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':active' => $params->active ? 1 : 0,

                        ':coupon_code' => $params->couponCode

                    );

        $pre->execute($values);



        // log action

        $act = $params->active ? 'activated' : 'deactivated';

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user ' . $act . ' the coupon code ' . $params->couponCode . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/togglerep', function($request, $response) {

    try {



        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        $sql = "update tblRepresentative set active = :active where rep_code = :rep_code";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':active' => $params->active ? 1 : 0,

                        ':rep_code' => $params->repCode

                    );

        $pre->execute($values);



        // log action

        $act = $params->active ? 'activated' : 'deactivated';

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user ' . $act . ' the representative ' . $params->repCode . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/admin/changepwd', function($request, $response) {

    try {



        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $con = $this->apidb;



        $sql = "update tblUser set password = :password where user_name = :user_name";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':password' => password_hash ( $params->pwd, PASSWORD_DEFAULT ),

                        ':user_name' => $params->userName

                    );

        $pre->execute($values);



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user changed the password of the user ' . $params->userName . '.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true', 'token' => $request->getAttribute('token') ), 200

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/userlist', function($request, $response) {

    try {

        $date = explode(';', $request->getAttribute('date'));

        $con = $this->apidb;

        $sql = "select u.user_name userName, first_name firstName, last_name lastName, concat(first_name, ' ', last_name) fullName, 

                       last_login lastLogin, creation_date creationDate, email, active, manage_order manageOrder, manage_user manageUser,

                       manage_stylist manageStylist, manage_referrals manageReferrals, manage_coupon manageCoupons, view_log viewLog, lcs, view_reports viewReports

                from tblUser u, tblUserPrivileges up

                where u.user_name = up.user_name";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user viewed the user list.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/replist/{type}', function($request, $response) {

    try {

        $con = $this->apidb;

        $sql = "select rep_code repCode, rep_name repName, active

                from tblRepresentative

                where type = :type";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

            ':type' => $request->getAttribute('type')

        );

        $pre->execute( $values );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user viewed the representative list.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/couponlist', function($request, $response) {

    try {

        $con = $this->apidb;

        $sql = "select coupon_code couponCode, created_date createdDate, created_by createdBy, valid_from validFrom, valid_to validTo, 

                       discount_amount discountAmount, discount_type discountType, active, balance

                from tblCoupon";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user viewed the coupon code list.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => 'true','data'=> $result ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->get('/log', function($request, $response) {

    try {

        $con = $this->apidb;

        $sql = "select DATE_FORMAT(log_date, '%Y-%m-%d @ %H:%i %p') logDate, order_id orderId, user_name userName, action, message, ip

                from tblLog

                order by log_date desc

                limit 200";

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );

        $result = $pre->fetchall();



        // log action

        $log = new LogQueries();

        $token = (new Parser())->parse($request->getAttribute('token'));

        $sql = $log->AdminAction($token->getClaim('usr'), 'The user viewed the log.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        return $response->withJson(array('status' => true,'data'=> $result ), 200);

    }

    catch(\Exception $ex){

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

})->add($mw_authorize);



$app->post('/signin', function($request, $response) {

    try {

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->cred;



        $usr = $params->userName;

        $pwd = $params->password;



        if ( strlen($usr) == 0 || strlen($pwd) == 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'Both username and password are required to sign in.' ), 200 );



        $hashed_pwd = password_hash ( $pwd, PASSWORD_DEFAULT );



        $con = $this->apidb;

        $sql = "select * from tblUser where user_name = :user_name and active = 1";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':user_name' => $usr

                    );

        $pre->execute($values);

        $userVals = $pre->fetchall();



        // If no account found

        if ( count($userVals) == 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'Username or password is incorrect.' ), 200 );

        if ( !password_verify($pwd, $userVals[0]['password'] ) )

            return $response->withJson( array('status' => 'false', 'message' => 'Username or password is incorrect.' ), 200 );



        // Get user privileges

        $sql = "select manage_order mngOdr, manage_user mngUsr, manage_stylist mngStl, manage_referrals mngRef, view_log vwLog, lcs, view_reports vwRpt, manage_coupon mngCpn

                from tblUserPrivileges where user_name = :user_name";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':user_name' => $usr

                    );

        $pre->execute($values);

        $result = $pre->fetch();



        // Update user last login

        $sql = "update tblUser set last_login = NOW() where user_name = :user_name";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':user_name' => $usr

                    );

        $pre->execute($values);



        // Generate JWT

        $signer = new Sha256();

        $token = (new Builder())->setIssuer('https://m2mkiosk.leonardofifthavenue.com') // Configures the issuer (iss claim)

                        ->setAudience('https://m2mkiosk.leonardofifthavenue.com/admin') // Configures the audience (aud claim)
                        ->setId('4f1g23a12aa', true) // Configures the id (jti claim), replicating as a header item

                        ->setIssuedAt(time()) // Configures the time that the token was issue (iat claim)

                        ->setNotBefore(time()) // Configures the time that the token can be used (nbf claim)

                        ->setExpiration(time() + 900 ) // Configures the expiration time of the token (exp claim)

                        ->set('usr', $usr) // Configures a new claim, called "username"

                        ->set('prv', $result) // Configures a new claim, called "privileges"

                        ->sign($signer, 'Lf@-m2M-K10Zk') // creates a signature using "testing" as key

                        ->getToken(); // Retrieves the generated token



        // log action

        $log = new LogQueries();

        $sql = $log->AdminLogin($usr, $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => 'true',

                  'token' => $token->__toString(),

                  'privileges' => $result,

                  'name' => $userVals[0]['first_name'] . ' ' . $userVals[0]['last_name'],

                  'last_login' => $userVals[0]['last_login'] ), 200 

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('error' => $ex->getMessage()), 422);

    }

});



$app->get('/admin/valusr', function($request, $response) {

    return $response->withJson(

        array('status' => 'true',

              'token' => $request->getAttribute('token') ), 200 

    );

})->add($mw_authorize);



$app->post('/valstr', function($request, $response) {

    try {

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $store_id = $params->storeId;

        $pwd = $params->password;



        if ( $store_id < 1 || strlen($pwd) == 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'You must select a store and enter the password.' ), 200 );



        $hashed_pwd = password_hash ( $pwd, PASSWORD_DEFAULT );



        $con = $this->apidb;

        $sql = "select password from tblStore where id = :store_id and active = 1";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

                        ':store_id' => $store_id

                    );

        $pre->execute($values);

        $result = $pre->fetchall();



        // If no account found

        if ( count($result) == 0 )

            return $response->withJson( array('status' => false, 'message' => 'The password is incorrect.' ), 200 );

        if ( !password_verify($pwd, $result[0]['password'] ) )

            return $response->withJson( array('status' => false, 'message' => 'The password is incorrect.' ), 200 );



        // Return success response code

        return $response->withJson(

            array('status' => true), 200 

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('status' => false, 'error' => $ex->getMessage()), 422);

    }

});



$app->post('/signlcs', function($request, $response) {

    try {

        // Get all the request values

        // $params = getOrderParams( $request );

        $params = json_decode( $request->getParsedBody()['form'] )->form;



        $pwd = $params->password;



        if ( strlen($pwd) == 0 )

            return $response->withJson( array('status' => 'false', 'message' => 'You must enter the password.' ), 200 );



        $con = $this->apidb;

        $sql = "select password from tblUser where user_name = 'leonardo' and active = 1";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute();

        $result = $pre->fetchall();



        // If no account found

        if ( count($result) == 0 )

            return $response->withJson( array('status' => false, 'message' => 'The password is incorrect.' ), 200 );

        if ( !password_verify($pwd, $result[0]['password'] ) )

            return $response->withJson( array('status' => false, 'message' => 'The password is incorrect.' ), 200 );



        // log action

        $log = new LogQueries();

        $sql = $log->AdminAction('lcorder', 'Leonardo Corder logged in from the kiosk.', $request->getAttribute('ip_address'), '', '');

        $pre = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $pre->execute( );



        // Return success response code

        return $response->withJson(

            array('status' => true), 200 

        );



    }

    catch( \Exception $ex ) {

        return $response->withJson(array('status' => false, 'error' => $ex->getMessage()), 422);

    }

});



$app->get('/odrpmtsmy/{orderId}/{attribute}', function($request, $response) {

    try {

        $orderId = $request->getAttribute('orderId');

        $attribute = $request->getAttribute('attribute');



        // Validate user input

        if ( strlen( $orderId ) < 4 )

            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'You must enter a valid order number.'), 200);

        if ( strlen( $attribute ) < 1 )

            return $response->withJson(array('status' => false, 'error' => true, 'message' => 'You must enter the email or phone number associated with your order.'), 200);



        // Get order details

        $con = $this->apidb;

        $sql = "select order_id, DATE_FORMAT(order_date, '%m-%d-%Y @ %H:%i %p') order_date, concat(customer_name, ' ', customer_lastname) customer_fullname, total, o.status, total - (select ifnull(sum(payment_amount),0) from tblOrderPayment op where op.order_id = o.order_id) balance 

                from tblOrder o

                where order_id = :order_id and (customer_phone = :attribute or customer_email = :attribute)";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

            ':order_id'    => $orderId,

            ':attribute'   => $attribute,

        );

        $pre->execute( $values );

        $order = $pre->fetchall();



        if( count( $order ) < 1 )

            return $response->withJson(array('status' => false, 'error' => false, 'message' => 'No order found with the given information.'), 200);



        $sql = "select DATE_FORMAT(p.payment_date, '%m-%d-%Y @ %H:%i %p') payment_date, p.payment_amount, p.memo, p.payment_method, p.payment_confirmation_number 

                from tblOrder o, tblOrderPayment p

                where o.order_id = p.order_id and o.order_id = :order_id

                order by payment_date";

        $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

        $values = array(

            ':order_id' => $orderId

        );

        $pre->execute( $values );

        $payments = $pre->fetchall();



        $order[0]['payments'] = $payments;

        return $response->withJson(array('status' => true, 'data' => $order[0]), 200);

    }

    catch( \Exception $ex ) {

        return $response->withJson(array('status' => false, 'error' => $ex->getMessage()), 422);

    }

});



function registerNote( $params, $con, $usr ) { 

    // Change order status

    $sql = "insert into `tblOrderNote` (order_id, note_date, author, message) values (:order_id, now(), :author, :message)";

    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

    $values = array(

                    ':order_id' => $params->orderId,

                    ':author' => $usr,

                    ':message' => $params->message

                );

    $pre->execute($values);



    $sql = "select n.order_id, cast(note_date as date) note_date, TIME_FORMAT(cast(note_date as time), '%H:%i %p') note_time, message, concat(first_name, ' ', last_name) author 

            from tblOrderNote n, tblUser u 

            where n.author = u.user_name and n.author = :author and n.order_id = :order_id

            order by n.note_date desc";

    $pre  = $con->prepare($sql, array(PDO::ATTR_CURSOR => PDO::CURSOR_FWDONLY));

    $values = array(

                    ':order_id' => $params->orderId,

                    ':author' => $usr

                );

    $pre->execute($values);

    $note = $pre->fetch();



    // Return Order information

    return array(

                'orderId' => $note['order_id'], 

                'noteDate' => $note['note_date'],

                'noteTime' => $note['note_time'],

                'author' => $note['author'],

                'message' => $note['message']

    );

}



function getOrderParams( $request ) {

    $postVars = json_decode( $request->getParsedBody()['form'] )->form;

    $params = new \stdClass();

    $params->customer_name = $postVars['customerName'];

    $params->customer_email = $postVars['customerEmail'];

    $params->customer_phone = $postVars['customerPhone'];

    $params->customer_measurements = new \stdClass();

    $params->customer_measurements->height_feet = $postVars['customerMeasurements']['heightFeet'];

    $params->customer_measurements->height_inches = $postVars['customerMeasurements']['heightInches'];

    $params->customer_measurements->neck = $postVars['customerMeasurements']['neck'];

    $params->customer_measurements->shoulder = $postVars['customerMeasurements']['shoulder'];

    $params->customer_measurements->sleeve_length = $postVars['customerMeasurements']['sleeve_length'];

    $params->customer_measurements->bicep = $postVars['customerMeasurements']['bicep'];

    $params->customer_measurements->wrist = $postVars['customerMeasurements']['wrist'];

    $params->customer_measurements->chest = $postVars['customerMeasurements']['chest'];

    $params->customer_measurements->stomach = $postVars['customerMeasurements']['stomach'];

    $params->customer_measurements->jacket_length_front = $postVars['customerMeasurements']['jacketLengthFront'];

    $params->customer_measurements->jacket_length_back = $postVars['customerMeasurements']['jacketLengthBack'];

    $params->customer_measurements->waist = $postVars['customerMeasurements']['waist'];

    $params->customer_measurements->hips = $postVars['customerMeasurements']['hips'];

    $params->customer_measurements->thigh = $postVars['customerMeasurements']['thigh'];

    $params->customer_measurements->crotch = $postVars['customerMeasurements']['crotch'];

    $params->customer_measurements->knee = $postVars['customerMeasurements']['knee'];

    $params->customer_measurements->leg_length = $postVars['customerMeasurements']['legLength'];

    $params->customer_measurements->watch_hand = $postVars['customerMeasurements']['watchHand'];

    $params->customer_measurements->vest_length_front = $postVars['customerMeasurements']['vestFront'];

    $params->customer_measurements->vest_length_back = $postVars['customerMeasurements']['vestBack'];

    $params->customer_measurements->cuff = $postVars['customerMeasurements']['cuff'];

    $params->rep = $postVars['rep'];

    $params->items = $postVars['items'];

    $params->subtotal = $postVars['subtotal'];

    $params->tax = $postVars['tax'];

    $params->total = $postVars['total'];

    $params->payment_amount = $postVars['paymentAmount'];

    $params->coupon_code = $postVars['couponCode'];

    $params->discount_amount = $postVars['discountAmount'];

    $params->payment_token = $postVars['paymentToken'];



    return $params;

}



function validateOrderParameters( $params ) {

    $errors = array();



    if( strlen( $params->paymentToken ) == 0 && $params->paymentMethod == 'Stripe' ) 

        array_push( $errors, 'There was an error processing the payment. Make sure the payment details are correct.' );



    if( strlen($params->customerName) < 3 ) 

        array_push( $errors, 'The customer first name is required and must be valid.' );



    if( strlen($params->customerLastName) < 3 ) 

        array_push( $errors, 'The customer last name is required and must be valid.' );



    if( strlen($params->customerGender) < 1 ) 

        array_push( $errors, 'The customer gender is required and must be valid.' );



    $arr_birthdate = explode( '-', $params->customerBirthdate );

    if( !checkdate($arr_birthdate[1], $arr_birthdate[2], $arr_birthdate[0]) ) 

        array_push( $errors, 'The customer birth date is required and must be valid.' );



    if( strlen($params->customerAddressLine1) < 3 ) 

        array_push( $errors, 'The customer address line 1 is required and must be valid.' );



    if( strlen($params->customerState) < 2 ) 

        array_push( $errors, 'The customer address state is required and must be valid.' );



    if( strlen($params->customerZipcode) < 5 ) 

        array_push( $errors, 'The customer address zip code is required and must be valid.' );

    

    preg_match('/.+@.+/', $params->customerEmail, $matches);

    if( count( $matches ) == 0 ) {

        array_push( $errors, 'The customer email is required and must be valid.' );

    }



    preg_match('/^[+]*[(]{0,1}[0-9]{1,4}[)]{0,1}[-\s\.0-9]*$/', $params->customerPhone, $matches);

    if( count( $matches ) == 0 ) {

        array_push( $errors, 'The customer phone is required and must be valid.' );

    }



    preg_match('/^[0-9]{1}$/', $params->customerMeasurements->heightFeet, $matches);

    if( count( $matches ) == 0 ) {

        array_push( $errors, 'The height - feet is required and must be valid.' );

    }

    if( !( intval( $params->customerMeasurements->heightInches ) >= 0 &&

        intval( $params->customerMeasurements->heightInches ) <= 11 ) ) {

        array_push( $errors, 'The height - inches is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->neck ) ) {

        array_push( $errors, 'The neck measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->shoulder ) ) {

        array_push( $errors, 'The shoulder measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->sleeveLengthLeft ) ) {

        array_push( $errors, 'The sleeve length left measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->sleeveLengthRight ) ) {

        array_push( $errors, 'The sleeve length right measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->coatSleeveLengthLeft ) ) {

        array_push( $errors, 'The coat sleeve length left measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->coatSleeveLengthRight ) ) {

        array_push( $errors, 'The coat sleeve length right measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->bicep ) ) {

        array_push( $errors, 'The bicep measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->wrist ) ) {

        array_push( $errors, 'The wrist measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->chest ) ) {

        array_push( $errors, 'The chest measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->stomach ) ) {

        array_push( $errors, 'The stomach measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->jacketLengthFront ) ) {

        array_push( $errors, 'The jacket length front measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->jacketLengthBack ) ) {

        array_push( $errors, 'The jacket length back measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->waist ) ) {

        array_push( $errors, 'The waist measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->hips ) ) {

        array_push( $errors, 'The hips measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->thigh ) ) {

        array_push( $errors, 'The thigh measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->crotch ) ) {

        array_push( $errors, 'The crotch measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->knee ) ) {

        array_push( $errors, 'The knee measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->legLength ) ) {

        array_push( $errors, 'The leg length measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->vestFront ) ) {

        array_push( $errors, 'The vest front measurement is required and must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->vestBack ) ) {

        array_push( $errors, 'The vest back measurement is required and must be valid.' );

    }

    if( strlen( $params->customerMeasurements->calf ) > 0 && !is_numeric( $params->customerMeasurements->calf ) ) {

        array_push( $errors, 'The calf measurement must be valid.' );

    }

    if( !is_numeric( $params->customerMeasurements->cuff ) ) {

        array_push( $errors, 'The cuff measurement is required and must be valid.' );

    }

    if( strtolower( $params->customerMeasurements->watchHand ) != 'right' &&

        strtolower( $params->customerMeasurements->watchHand ) != 'left' &&

        strtolower( $params->customerMeasurements->watchHand ) != 'none' ) {

        array_push( $errors, 'The watch hand is required and must be valid.' );

    }



    if( strlen( $params->rep ) == 0 ) {

        array_push( $errors, 'The representative code is required and must be valid.' );

    }

    if( !is_numeric( $params->subtotal ) ) {

        array_push( $errors, 'There is something wrong with the order subtotal.' );

    }

    if( !is_numeric( $params->tax ) ) {

        array_push( $errors, 'There is something wrong with the order tax.' );

    }

    if( !is_numeric( $params->total ) ) {

        array_push( $errors, 'There is something wrong with the order total.' );

    }

    if( !is_numeric( floatval( $params->payment_amount ) ) ) {

        array_push( $errors, 'There is something wrong with the order payment amount.' );

    }

    // if( floatval( str_replace(',', '', $params->paymentAmount ) ) < round( floatval( $params->total ) * 0.25, 2 ) ) {

    //     array_push( $errors, 'There is a minimum payment amount of 25% of the order. ' );

    // }

    if( $params->paymentMethod != 'Stripe' && $params->paymentConfirmationNumber == '' ) {

        array_push( $errors, 'The payment confirmation number is required.' );

    }

    if( strlen( $params->store ) == 0 ) {

        array_push( $errors, 'The store is required.' );

    }



    return $errors;

}



function sendEmail ( $type, $user_full_name, $to_email, $order_id, $status = '' ) {



    $mail = new PHPMailer(true);

    try {

        // Server settings

        // $mail->SMTPDebug = 2;                                 // Enable verbose debug output

        $mail->isSMTP();                                      // Set mailer to use SMTP

        $mail->Host = 'mail.leonardofifthavenue.com';         // Specify main and backup SMTP servers

        $mail->SMTPAuth = false;                              // Enable SMTP authentication

        // $mail->Username = 'user@example.com';                 // SMTP username

        // $mail->Password = 'secret';                           // SMTP password

        // $mail->SMTPSecure = 'tls';                            // Enable TLS encryption, `ssl` also accepted

        $mail->Port = 26;                                     // TCP port to connect to



        // Recipients

        $mail->setFrom('customerservice@leonardofifthavenue.com', 'LFA - Design to Measure');

        $mail->addAddress($to_email, $user_full_name);     // Add a recipient



        // Attachments

        // $mail->addAttachment('/var/tmp/file.tar.gz');         // Add attachments

        // $mail->addAttachment('/tmp/image.jpg', 'new.jpg');    // Optional name



        // Content

        $mail->isHTML(true);                                  // Set email format to HTML



        switch( $type ) {

            case 'lcs.neworder':

                mailBodyLCSNewOrder( $mail, $user_full_name, $order_id, 'https://m2mkiosk.leonardofifthavenue.com/admin/#/orders' );

                break;

            case 'admin.orderapproved':

                mailBodyAdminNewOrder( $mail, $user_full_name, $order_id, 'https://m2mkiosk.leonardofifthavenue.com/admin/#/orders' );

                break;

            case 'client.neworder':

                mailBodyClientNewOrder( $mail, $user_full_name, $order_id, 'https://m2mkiosk.leonardofifthavenue.com/pay' );

                break;

            case 'client.statuschange':

                mailBodyClientOrderStatusChange( $mail, $user_full_name, $order_id, 'https://m2mkiosk.leonardofifthavenue.com/pay', $status );

                break;

        }



        // Send

        $mail->send();

        return 'Message has been sent';

    }

    catch ( Exception $ex ) {

        return 'Message could not be sent. Mailer Error: ' . $mail->ErrorInfo;

    }

}

function sendCustomerEmail( $to_email, $user_full_name, $subject, $message) {
    $mail = new PHPMailer(true);
    try {
        // Server settings
        // $mail->SMTPDebug = 2;                                 // Enable verbose debug output
        $mail->isSMTP();                                      // Set mailer to use SMTP
        $mail->Host = 'mail.leonardofifthavenue.com';         // Specify main and backup SMTP servers
        $mail->SMTPAuth = false;                              // Enable SMTP authentication
        // $mail->Username = 'user@example.com';                 // SMTP username
        // $mail->Password = 'secret';                           // SMTP password
        // $mail->SMTPSecure = 'tls';                            // Enable TLS encryption, `ssl` also accepted
        $mail->Port = 26;                                     // TCP port to connect to

        // Recipients
        $mail->setFrom('customerservice@leonardofifthavenue.com', 'LFA - Design to Measure');
        $mail->addAddress($to_email, $user_full_name);     // Add a recipient

        // Attachments
        // $mail->addAttachment('/var/tmp/file.tar.gz');         // Add attachments
        // $mail->addAttachment('/tmp/image.jpg', 'new.jpg');    // Optional name

        // Content
        $mail->isHTML(true);                                  // Set email format to HTML

        $mail->Subject = $subject;
        $mail->Body    = file_Get_contents('./email_templates/customer_message.txt');

        $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);
        $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);
        $mail->Body    = str_replace('*|MESSAGE|*', $message, $mail->Body);
        $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);

        // Send
        $mail->send();
        return 'Message has been sent';
    }
    catch ( Exception $ex ) {
        return 'Message could not be sent. Mailer Error: ' . $mail->ErrorInfo;
    }
}

function mailBodyLCSNewOrder( &$mail, $user_full_name, $order_id, $btn_link ) {

    $mail->Subject = 'Design to Measure Order #' . $order_id . ' waiting for approval';

    $mail->Body    = file_Get_contents('./email_templates/lcs_new_order.txt');



    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);

    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);

    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);

    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);

}



function mailBodyAdminNewOrder( &$mail, $user_full_name, $order_id, $btn_link ) {

    $mail->Subject = 'Design to Measure Order #' . $order_id . ' has been approved by designer';

    $mail->Body    = file_Get_contents('./email_templates/admin_new_order.txt');



    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);

    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);

    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);

    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);

}



function mailBodyClientNewOrder( &$mail, $user_full_name, $order_id, $btn_link ) {

    $mail->Subject = 'Thank you for your Design to Measure Order!';

    $mail->Body    = file_Get_contents('./email_templates/customer_new_order.txt');



    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);

    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);

    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);

    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);

}



function mailBodyClientOrderStatusChange( &$mail, $user_full_name, $order_id, $btn_link, $status ) {

    $mail->Subject = 'Design to Measure Order #' . $order_id . ' status update';

    $mail->Body    = file_Get_contents('./email_templates/customer_status_changed.txt');



    $mail->Body    = str_replace('*|MC:SUBJECT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|MC_PREVIEW_TEXT|*', $mail->Subject, $mail->Body);

    $mail->Body    = str_replace('*|FNAME|*', $user_full_name, $mail->Body);

    $mail->Body    = str_replace('*|ORDER_ID|*', $order_id, $mail->Body);

    $mail->Body    = str_replace('*|STATUS|*', $status, $mail->Body);

    $mail->Body    = str_replace('*|UPDATE_PROFILE|*', $btn_link, $mail->Body);

    $mail->Body    = str_replace('*|CURRENT_YEAR|*', date("Y"), $mail->Body);

}



$app->run();